@@ -21,6 +21,14 @@
 #include <cstdio>
 #include <stdexcept>
 
+#ifdef PQXX_HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#else
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
+#endif	// PQXX_HAVE_SYS_SELECT_H
+
 #include "pqxx/connection_base"
 #include "pqxx/result"
 #include "pqxx/transaction"
@@ -46,8 +54,10 @@ pqxx::connection_base::connection_base(const string &ConnInfo) :
   m_Conn(0),
   m_Trans(),
   m_Noticer(),
-  m_Trace(0)
+  m_Trace(0),
+  m_fdmask()
 {
+  clear_fdmask();
 }
 
 
@@ -56,8 +66,10 @@ pqxx::connection_base::connection_base(const char ConnInfo[]) :
   m_Conn(0),
   m_Trans(),
   m_Noticer(),
-  m_Trace(0)
+  m_Trace(0),
+  m_fdmask()
 {
+  clear_fdmask();
 }
 
 
@@ -101,8 +113,7 @@ void pqxx::connection_base::halfconnect()
 }
 
 
-void pqxx::connection_base::set_variable(const PGSTD::string &Var,
-					 const PGSTD::string &Value)
+void pqxx::connection_base::set_variable(const string &Var, const string &Value)
 {
   if (m_Trans.get())
   {
@@ -118,6 +129,23 @@ void pqxx::connection_base::set_variable(const PGSTD::string &Var,
 }
 
 
+string pqxx::connection_base::get_variable(const string &Var)
+{
+  return m_Trans.get() ? m_Trans.get()->get_variable(Var) : RawGetVar(Var);
+}
+
+
+string pqxx::connection_base::RawGetVar(const string &Var)
+{
+  // Is this variable in our local map of set variables?
+  // TODO: Is it safe to read-allocate variables in the "cache?"
+  map<string,string>::const_iterator i=m_Vars.find(Var);
+  if (i != m_Vars.end()) return i->second;
+
+  return Exec(("SHOW " + Var).c_str(), 0).at(0).at(0).as(string());
+}
+
+
 /** Set up various parts of logical connection state that may need to be
  * recovered because the physical connection to the database was lost and is
  * being reset, or that may not have been initialized yet.
@@ -327,9 +355,7 @@ const char *pqxx::connection_base::ErrMsg() const
 }
 
 
-pqxx::result pqxx::connection_base::Exec(const char Query[], 
-                                       int Retries, 
-				       const char OnReconnect[])
+pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
   activate();
 
@@ -338,23 +364,22 @@ pqxx::result pqxx::connection_base::Exec(const char Query[],
   while ((Retries > 0) && !R && !is_open())
   {
     Retries--;
-
-    Reset(OnReconnect);
+    Reset();
     if (is_open()) R = PQexec(m_Conn, Query);
   }
 
   // TODO: Is this really right?  What if there's just no memory?
   if (!R) throw broken_connection();
   R.CheckStatus(Query);
-
   get_notifs();
-
   return R;
 }
 
 
-void pqxx::connection_base::Reset(const char OnReconnect[])
+void pqxx::connection_base::Reset()
 {
+  clear_fdmask();
+
   // Forget about any previously ongoing connection attempts
   dropconnect();
 
@@ -367,20 +392,14 @@ void pqxx::connection_base::Reset(const char OnReconnect[])
   {
     PQreset(m_Conn);
     SetupState();
-
-    // Perform any extra patchup work involved in restoring the connection,
-    // typically set up a transaction.
-    if (OnReconnect)
-    {
-      result Temp( PQexec(m_Conn, OnReconnect) );
-      Temp.CheckStatus(OnReconnect);
-    }
+    clear_fdmask();
   }
 }
 
 
 void pqxx::connection_base::close() throw ()
 {
+  clear_fdmask();
   try
   {
     if (m_Trans.get()) 
@@ -407,6 +426,7 @@ void pqxx::connection_base::close() throw ()
   catch (...)
   {
   }
+  clear_fdmask();
 }
 
 
@@ -460,12 +480,12 @@ void pqxx::connection_base::MakeEmpty(pqxx::result &R, ExecStatusType Stat)
 }
 
 
-void pqxx::connection_base::BeginCopyRead(const string &Table)
+#ifndef PQXX_HAVE_PQPUTCOPY
+namespace
 {
-  const string CQ("COPY " + Table + " TO STDOUT");
-  result R( Exec(CQ.c_str()) );
-  R.CheckStatus(CQ);
+const string theWriteTerminator = "\\.";
 }
+#endif
 
 
 bool pqxx::connection_base::ReadCopyLine(string &Line)
@@ -474,6 +494,7 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
     throw logic_error("libpqxx internal error: "
 	              "ReadCopyLine() without connection");
 
+  Line.erase();
   bool Result;
 
 #ifdef PQXX_HAVE_PQPUTCOPY
@@ -484,10 +505,8 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
       throw runtime_error("Reading of table data failed: " + string(ErrMsg()));
 
     case -1:
-      {
-	result R(PQgetResult(m_Conn));
+      for (result R(PQgetResult(m_Conn)); R; R=PQgetResult(m_Conn))
 	R.CheckStatus("[END COPY]");
-      }
       Result = false;
       break;
 
@@ -496,21 +515,23 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 	  "table read inexplicably went asynchronous");
 
     default:
-      PQAlloc<char> PQA(Buf);
-      Line = Buf;
+      if (Buf)
+      {
+        PQAlloc<char> PQA(Buf);
+        Line = Buf;
+      }
       Result = true;
   }
 #else
   char Buf[10000];
   bool LineComplete = false;
 
-  Line.erase();
-
   while (!LineComplete)
   {
     switch (PQgetline(m_Conn, Buf, sizeof(Buf)))
     {
     case EOF:
+      PQendcopy(m_Conn);
       throw runtime_error("Unexpected EOF from backend");
 
     case 0:
@@ -527,88 +548,139 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
     Line += Buf;
   }
 
-  Result = (Line != "\\.");
+  Result = (Line != theWriteTerminator);
 
-  if (!Result) Line.erase();
+  if (!Result) 
+  {
+    Line.erase();
+    if (PQendcopy(m_Conn)) throw runtime_error(ErrMsg());
+  }
 #endif
 
   return Result;
 }
 
 
-void pqxx::connection_base::BeginCopyWrite(const string &Table)
-{
-  const string CQ("COPY " + Table + " FROM STDIN");
-  result R( Exec(CQ.c_str()) );
-  R.CheckStatus(CQ);
-}
-
-
-
-void pqxx::connection_base::WriteCopyLine(const string &Line)
+#include<iostream>// DEBUG CODE
+bool pqxx::connection_base::WriteCopyLine(const string &Line, bool async)
 {
   if (!is_open())
     throw logic_error("libpqxx internal error: "
 	              "WriteCopyLine() without connection");
 
-  // TODO: Check for errors!!!
+  bool OK, Result;
+  int PutRes;
+  const string L = Line + '\n';
+  const char *const LC = L.c_str();
+  const string::size_type Len = L.size();
+
 #ifdef PQXX_HAVE_PQPUTCOPY
-  PQputCopyData(m_Conn, (Line + "\n").c_str(), Line.size()+1);
+  if (async) go_async();
+  PutRes = PQputCopyData(m_Conn, LC, Len);
+  if (async) go_sync();
+  OK = (PutRes != -1);
+  Result = (PutRes != 0);
 #else
-  PQputline(m_Conn, (Line + "\n").c_str());
+  PutRes = PQputnbytes(m_Conn, LC, Len);
+  OK = (PutRes != EOF);
+  Result = true;
 #endif
+  if (!OK)
+  {
+    const string Msg = string("Error writing to table: ") + ErrMsg();
+#ifdef PQXX_HAVE_PQPUTCOPY
+    PQendcopy(m_Conn);
+#endif
+    throw runtime_error(Msg);
+  }
+  return Result;
 }
 
 
-void pqxx::connection_base::EndCopyWrite() throw ()
+void pqxx::connection_base::EndCopyWrite()
 {
 #ifdef PQXX_HAVE_PQPUTCOPY
-  switch (PQputCopyEnd(m_Conn, NULL))
+  go_sync();
+  int Res;
+  do
   {
-    case -1:
-      process_notice("Write to table failed: " + string(ErrMsg()) + "\n");
-      break;
-    case 0:
-      process_notice("libpqxx internal error: table write went asynchronous\n");
-      break;
-    case 1:
-      try
-      {
-        result R(PQgetResult(m_Conn));
-	R.CheckStatus("[END COPY]");
-      }
-      catch (const exception &e)
-      {
-	process_notice(string(e.what()) + "\n");
-      }
-  }
-
+    Res = PQputCopyEnd(m_Conn, NULL);
+    switch (Res)
+    {
+      case -1:
+        throw runtime_error("Write to table failed: " + string(ErrMsg()));
+      case 0:
+        throw logic_error("libpqxx internal error: "
+	    "table write is inexplicably asynchronous");
+      case 1:
+        for (result R(PQgetResult(m_Conn)); R; R=PQgetResult(m_Conn))
+	    R.CheckStatus("[END COPY]");
+        break;
+      default:
+        throw logic_error("libpqxx internal error: "
+	    "unexpected result " + ToString(Res) + " from PQputCopyEnd()");
+    }
+  } while (!Res);
 #else
-  try
-  {
-    static const string terminator("\\.");
-    WriteCopyLine(terminator);
-  }
-  catch (const exception &e)
-  {
-    // TODO: Set some flag so transaction commit will be sure to fail?
-    process_notice(e.what());
-  }
+  WriteCopyLine(theWriteTerminator);
+  // This check is a little odd, but for some reason PostgreSQL 7.4 keeps
+  // returning 1 (i.e., failure) but with an empty error message, and without
+  // anything seeming wrong.
+  if ((PQendcopy(m_Conn) != 0) && ErrMsg() && *ErrMsg()) 
+    throw runtime_error(ErrMsg());
 #endif
 }
 
-#ifndef PQXX_HAVE_PQPUTCOPY
-// End COPY operation.  Careful: this assumes that no more lines remain to be
-// read, or (respectively) that the write operation has been terminated with a
-// closing line.
-void pqxx::connection_base::EndCopy() throw ()
+
+namespace
 {
-  // Not needed for the new interface.
-  // Don't throw exception on failure--this code is only called from TableStream
-  // destructors.
-  PQendcopy(m_Conn);
+#ifdef PQXX_SELECT_ACCEPTS_NULL
+  // The always-empty fd_set
+  fd_set *const fdset_none = 0;
+#else	// PQXX_SELECT_ACCEPTS_NULL
+  fd_set emptyfd;	// Relies on zero-initialization
+  fd_set *const fdset_none = &emptyfd;
+#endif	// PQXX_SELECT_ACCEPTS_NULL
+} // namespace
+
+
+int pqxx::connection_base::set_fdmask() const
+{
+  const int fd = PQsocket(m_Conn);
+  if (fd < 0) throw broken_connection();
+  FD_SET(fd, &m_fdmask);
+  return fd;
 }
-#endif
 
 
+void pqxx::connection_base::clear_fdmask() throw ()
+{
+  FD_ZERO(&m_fdmask);
+}
+
+void pqxx::connection_base::wait_read() const
+{
+  const int fd = set_fdmask();
+  select(fd+1, &m_fdmask, fdset_none, &m_fdmask, 0);
+}
+
+void pqxx::connection_base::wait_write() const
+{
+  const int fd = set_fdmask();
+  select(fd+1, fdset_none, &m_fdmask, &m_fdmask, 0);
+}
+
+
+void pqxx::connection_base::go_sync()
+{
+  if (PQsetnonblocking(m_Conn, false)==-1)
+    throw runtime_error("Return to blocking mode failed: "+string(ErrMsg()));
+}
+
+void pqxx::connection_base::go_async()
+{
+  if (PQsetnonblocking(m_Conn, true)==-1)
+    throw runtime_error("Could not go to nonblocking mode: "+string(ErrMsg()));
+}
+
 
