@@ -30,6 +30,6 @@ pqxx::nontransaction::~nontransaction()
 
 pqxx::result pqxx::nontransaction::do_exec(const char Query[])
 {
-  return DirectExec(Query, 2, 0);
+  return DirectExec(Query, 2);
 }
 
