@@ -28,15 +28,33 @@ pqxx::tablestream::tablestream(transaction_base &STrans,
 			       const string &Null) :
   m_Trans(STrans),
   m_Name(SName),
-  m_Null(Null)
+  m_Null(Null),
+  m_Finished(false)
 {
-  STrans.RegisterStream(this);
 }
 
 
 pqxx::tablestream::~tablestream()
 {
-  m_Trans.UnregisterStream(this);
-  m_Trans.EndCopy();
 }
 
+
+void pqxx::tablestream::register_me()
+{
+  m_Trans.RegisterStream(this);
+}
+
+
+void pqxx::tablestream::base_close()
+{
+  m_Finished = true;
+  m_Trans.UnregisterStream(this);
+}
+
+
+void pqxx::tablestream::RegisterPendingError(const string &Err) throw ()
+{
+  m_Trans.RegisterPendingError(Err);
+}
+
+
