@@ -36,7 +36,8 @@ pqxx::transaction_base::transaction_base(connection_base &C,
   m_UniqueCursorNum(1),
   m_Stream(),
   m_Status(st_nascent),
-  m_Registered(false)
+  m_Registered(false),
+  m_PendingError()
 {
   m_Conn.RegisterTransaction(this);
   m_Registered = true;
@@ -45,17 +46,29 @@ pqxx::transaction_base::transaction_base(connection_base &C,
 
 pqxx::transaction_base::~transaction_base()
 {
-  if (m_Registered)
+  try
   {
-    m_Conn.process_notice("Transaction '" + m_Name + "' "
-		          "was never closed properly!\n");
-    m_Conn.UnregisterTransaction(this);
+    if (!m_PendingError.empty())
+      process_notice("UNPROCESSED ERROR: " + m_PendingError + "\n");
+
+    if (m_Registered)
+    {
+      m_Conn.process_notice("Transaction '" + m_Name + "' "
+		            "was never closed properly!\n");
+      m_Conn.UnregisterTransaction(this);
+    }
+  }
+  catch (const exception &e)
+  {
+    process_notice(string(e.what()) + "\n");
   }
 }
 
 
 void pqxx::transaction_base::commit()
 {
+  CheckPendingError();
+
   // Check previous status code.  Caller should only call this function if
   // we're in "implicit" state, but multiple commits are silently accepted.
   switch (m_Status)
@@ -164,12 +177,14 @@ void pqxx::transaction_base::abort()
 
 
 pqxx::result pqxx::transaction_base::exec(const char Query[],
-    					const string &Desc)
+    					  const string &Desc)
 {
-  const string N = (Desc.empty() ? "" : "'" + Desc + "'");
+  CheckPendingError();
+
+  const string N = (Desc.empty() ? "" : "'" + Desc + "' ");
 
   if (m_Stream.get())
-    throw logic_error("Attempt to execute query '" + N + " on transaction '" + 
+    throw logic_error("Attempt to execute query " + N + "on transaction '" + 
 		      name() + 
 		      "' while stream '" +
 		      m_Stream.get()->name() +
@@ -186,19 +201,19 @@ pqxx::result pqxx::transaction_base::exec(const char Query[],
     break;
 
   case st_committed:
-    throw logic_error("Attempt to execute query " + N + " "
+    throw logic_error("Attempt to execute query " + N +
 	              "in committed transaction '" +
 		      name() +
 		      "'");
 
   case st_aborted:
-    throw logic_error("Attempt to execute query " + N + " "
+    throw logic_error("Attempt to execute query " + N +
 	              "in aborted transaction '" +
 		      name() +
 		      "'");
 
   case st_in_doubt:
-    throw logic_error("Attempt to execute query " + N + " in transaction '" + 
+    throw logic_error("Attempt to execute query " + N + "in transaction '" + 
 		      name() + 
 		      "', which is in indeterminate state");
   default:
@@ -206,19 +221,28 @@ pqxx::result pqxx::transaction_base::exec(const char Query[],
 		      "invalid status code");
   }
 
-  // TODO: Pass Name to DoExec(), and from there on down
+  // TODO: Pass Desc to do_exec(), and from there on down
   return do_exec(Query);
 }
 
 
-void pqxx::transaction_base::set_variable(const PGSTD::string &Var,
-                                          const PGSTD::string &Value)
+void pqxx::transaction_base::set_variable(const string &Var,
+                                          const string &Value)
 {
+  // Before committing to this new value, see what the backend thinks about it
   m_Conn.RawSetVar(Var, Value);
   m_Vars[Var] = Value;
 }
 
 
+string pqxx::transaction_base::get_variable(const string &Var) const
+{
+  const map<string,string>::const_iterator i = m_Vars.find(Var);
+  if (i != m_Vars.end()) return i->second;
+  return m_Conn.RawGetVar(Var);
+}
+
+
 void pqxx::transaction_base::Begin()
 {
   if (m_Status != st_nascent)
@@ -251,6 +275,8 @@ void pqxx::transaction_base::End() throw ()
     m_Conn.UnregisterTransaction(this);
     m_Registered = false;
 
+    CheckPendingError();
+
     if (m_Stream.get())
       m_Conn.process_notice("Closing transaction '" +
 		            name() +
@@ -287,10 +313,63 @@ void pqxx::transaction_base::UnregisterStream(tablestream *S) throw ()
 }
 
 
-pqxx::result pqxx::transaction_base::DirectExec(const char C[], 
-		                      int Retries,
-				      const char OnReconnect[])
+pqxx::result pqxx::transaction_base::DirectExec(const char C[], int Retries)
 {
-  return m_Conn.Exec(C, Retries, OnReconnect);
+  CheckPendingError();
+  return m_Conn.Exec(C, Retries);
 }
 
+
+void pqxx::transaction_base::RegisterPendingError(const string &Err) throw ()
+{
+  if (m_PendingError.empty() && !Err.empty())
+  {
+    try
+    {
+      m_PendingError = Err;
+    }
+    catch (const exception &e)
+    {
+      try
+      {
+        process_notice("UNABLE TO PROCESS ERROR\n");
+        process_notice(e.what());
+        process_notice("ERROR WAS:");
+        process_notice(Err);
+      }
+      catch (...)
+      {
+      }
+    }
+  }
+}
+
+
+void pqxx::transaction_base::CheckPendingError()
+{
+  if (!m_PendingError.empty())
+  {
+    const string Err(m_PendingError);
+#ifdef PQXX_HAVE_STRING_CLEAR
+    m_PendingError.clear();
+#else
+    m_PendingError.resize(0);
+#endif
+    throw runtime_error(m_PendingError);
+  }
+}
+
+
+void pqxx::transaction_base::BeginCopyRead(const string &Table)
+{
+  exec("COPY " + Table + " TO STDOUT");
+}
+
+
+void pqxx::transaction_base::BeginCopyWrite(const string &Table)
+{
+  exec("COPY " + Table + " FROM STDIN");
+  m_Conn.go_async();
+}
+
+
